﻿using System;
using System.Configuration;
using CookComputing.XmlRpc;
using System.Collections.Generic;
using System.IO;
using System.Linq;

//50% of this code came from Keyvan Nayyeri's excellent blog post. 
// http://nayyeri.net/implement-metaweblog-api-in-asp-net. He rocks. Nice job.
// The XmlRpcService is the timeless work of Charles Cook. Also rocks and has for a long time.
// I mashed it up.
// Scott Hanselman - http://hanselman.com

using Models = $rootnamespace$.Models;

namespace $rootnamespace$.MetaWeblog
{
    public class MetaWeblog : XmlRpcService, IMetaWeblog
    {
        IPostRepository repo;

        public MetaWeblog(IPostRepository r)
        {
            repo = r;
        }
        
        string IMetaWeblog.AddPost(string blogid, string username, string password,
            Post post, bool publish)
        {
            if (ValidateUser(username, password))
            {
                var p = new Models.Post() { 
                        PublishDate = DateTime.Now, 
                        Title = post.title,
                        Text = post.description };
                repo.Add(p);
                repo.Save();
                return p.ID.ToString();
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        bool IMetaWeblog.UpdatePost(string postid, string username, string password,
            Post post, bool publish)
        {
            if (ValidateUser(username, password))
            {
                Models.Post p = repo.GetById(int.Parse(postid));
                p.PublishDate = DateTime.Now;
                p.Title = post.title;
                p.Text = post.description;
                repo.Save();
                return true;
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        Post IMetaWeblog.GetPost(string postid, string username, string password)
        {
            if (ValidateUser(username, password))
            {
                Models.Post p = repo.GetById(int.Parse(postid));
                Post post = new Post()
                {
                    description = p.Text,
                    title = p.Title,
                    postid = p.ID,
                    dateCreated = p.PublishDate
                };
                return post;
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        CategoryInfo[] IMetaWeblog.GetCategories(string blogid, string username, string password)
        {
            if (ValidateUser(username, password))
            {
                List<CategoryInfo> categoryInfos = new List<CategoryInfo>();
                // TODO: Implement your own logic to get category info and set the categoryInfos
                return categoryInfos.ToArray();
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        bool IMetaWeblog.SetCategories(string postid, string username, string password, CategoryInfo[] categories)
        {
            return true;
        }

        Post[] IMetaWeblog.GetRecentPosts(string blogid, string username, string password,
            int numberOfPosts)
        {
            if (ValidateUser(username, password))
            {
                var posts = (from p in repo.GetAllPosts() select 
                     new Post() {
                        description = p.Text,
                        title = p.Text,
                        postid = p.ID,
                        dateCreated = p.PublishDate
                    });

                return posts.ToArray();
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        MediaObjectInfo IMetaWeblog.NewMediaObject(string blogid, string username, string password,
            MediaObject mediaObject)
        {
            if (ValidateUser(username, password))
            {
                MediaObjectInfo objectInfo = new MediaObjectInfo();

                //TODO: Put a picture somewhere and return a URL...
                //File.WriteAllBytes(pathCombined, mediaObject.bits);
                //objectInfo.url = pathCombined;

                return objectInfo;
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        bool IMetaWeblog.DeletePost(string key, string postid, string username, string password, bool publish)
        {
            if (ValidateUser(username, password))
            {
                throw new XmlRpcFaultException(0, "Delete it yourself!");
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        BlogInfo[] IMetaWeblog.GetUsersBlogs(string key, string username, string password)
        {
            if (ValidateUser(username, password))
            {
                List<BlogInfo> infoList = new List<BlogInfo>();

                BlogInfo b = new BlogInfo();
                b.blogName = "My crappy blog";
                b.blogid = "My crappy blog";
                b.url = "";

                infoList.Add(b);

                return infoList.ToArray();
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        UserInfo IMetaWeblog.GetUserInfo(string key, string username, string password)
        {
            if (ValidateUser(username, password))
            {
                UserInfo info = new UserInfo();
                return info;
            }
            throw new XmlRpcFaultException(0, "User is not valid!");
        }

        private bool ValidateUser(string username, string password)
        {
            bool result = true;
            // TODO: Implement the logic to validate the user
            return result;
        }
    }
}